<?php

/**
 *
 * @link              https://agilestorelocator.com
 * @since             1.0.0
 * @package           AgileStoreLocator
 *
 * @wordpress-plugin
 * Plugin Name:       Agile Store Locator
 * Plugin URI:        https://agilestorelocator.com
 * Description:       Agile Store Locator is a WordPress Store Finder/Locator Plugin that renders stores list with Location markers on Google Maps v3, it supports GeoLocation and render nearest stores with direction over google maps.
 * Version:           1.4.2
 * Author:            AGILELOGIX
 * Author URI:        https://agilestorelocator.com/
 * License:           Copyrights 2022
 * License URI:       
 * Text Domain:       asl_locator
 * Domain Path:       /languages/
 */

if ( !class_exists( 'ASL_Store_locator' ) ) {


  class ASL_Store_locator {
        
        /**
         * Class constructor
         */          
        function __construct() {
                                    
            $this->define_constants();
            $this->includes();


            register_activation_hook( __FILE__, array( $this, 'activate') );
            register_deactivation_hook( __FILE__, array( $this, 'deactivate') );
        }
        
        /**
         * Setup plugin constants.
         *
         * @since 1.0.0
         * @return void
         */
        public function define_constants() {

          global $wpdb;

          $upload_dir       = wp_upload_dir();
          
          define( 'ASL_PLUGIN', 'agile-store-locator');
          define( 'ASL_URL_PATH', plugin_dir_url( __FILE__ ) );
          define( 'ASL_PLUGIN_PATH', plugin_dir_path(__FILE__) );
          define( 'ASL_BASE_PATH', dirname( plugin_basename( __FILE__ ) ) );
          define( 'ASL_PREFIX', $wpdb->prefix."asl_" );
          define( 'ASL_CVERSION', "1.4.2" );
          define( 'ASL_UPLOAD_DIR', $upload_dir['basedir'].'/'.ASL_PLUGIN.'/' );
          define( 'ASL_UPLOAD_URL', $upload_dir['baseurl'].'/'.ASL_PLUGIN.'/' );

          //  User Permission
          if ( ! defined( 'ASL_PERMISSION' ) ) {
            define('ASL_PERMISSION', 'edit_pages');
          }
        }
        
        /**
         * Include the required files.
         *
         * @since 1.0.0
         * @return void
         */
        public function includes() {

          require_once ASL_PLUGIN_PATH . 'includes/class-agile-store-locator.php';

          //  helper library
          require_once ASL_PLUGIN_PATH . 'includes/class-agile-store-locator-helper.php';
          
          $asl_core = new AgileStoreLocator();
          $asl_core->run();
        }
        

        /**
         * The code that runs during plugin activation.
         * This action is documented in includes/class-agile-store-locator-activator.php
         */
        public function activate() {
          
          require_once ASL_PLUGIN_PATH . 'includes/class-agile-store-locator-activator.php';
          AgileStoreLocator_Activator::activate();

          //  Copy the Assets to the uploads directory
          AgileStoreLocator_Helper::copy_assets();
        }

        /**
         * The code that runs during plugin deactivation.
         * This action is documented in includes/class-agile-store-locator-deactivator.php
         */
        public function deactivate() {
          
          require_once ASL_PLUGIN_PATH . 'includes/class-agile-store-locator-deactivator.php';
          AgileStoreLocator_Deactivator::deactivate();
        }
  }


  $asl_instance = new ASL_Store_locator();
}

