<?php


/**
 * Helper Class for the Store Locator
 */
class AgileStoreLocator_Helper {


  /**
   * [send_email Send a email notification]
   * @param  [type] $to_email [description]
   * @param  [type] $subject  [description]
   * @param  [type] $message  [description]
   * @return [type]           [description]
   */
  public static function send_email($to_email, $subject, $message, $add_headers = null) {

    $headers = array('Content-Type: text/html; charset=UTF-8');
    
    //  Merge headers
    if($add_headers && !empty($add_headers)) {
      $headers = array_merge($headers, $add_headers);
    } 
       
    return wp_mail($to_email, $subject, $message, $headers);
  }


  /**
   * [get_kml_files Get the list of the KML files]
   * @return [type] [description]
   */
  public static function get_kml_files() {

    $dir   = ASL_UPLOAD_DIR.'kml/';
    $files = [];

    $scan_results = @scandir($dir);
    
    if($scan_results) {

      $files = array_slice($scan_results, 2); 
    }

    return $files;
  }

  /**
   * [get_store Get the store by the store id or where clause]
   * @param  [type] $origLat [description]
   * @param  [type] $origLon [description]
   * @param  [type] $dist    [description]
   * @return [type]          [description]
   */
  public static function get_store($store_id, $_where_clause = null) {

    global $wpdb;
    
    $ASL_PREFIX   = ASL_PREFIX;


    $store_id     = ($store_id)? intval($store_id): '';

    $where_clause = ($_where_clause)? $_where_clause: "s.`id` = {$store_id}";

    //  Query the get the closest store
    $query   = "SELECT s.`id`, `title`,  `description`, `street`,  `city`,  `state`, `postal_code`, `lat`,`lng`,`phone`,  `fax`,`email`,`website`,`logo_id`,{$ASL_PREFIX}storelogos.`path`,`marker_id`,`description_2`,`open_hours`, `ordr`,`brand`,`special`, `custom`,`slug` FROM {$ASL_PREFIX}stores as s 
          LEFT JOIN {$ASL_PREFIX}storelogos ON logo_id = {$ASL_PREFIX}storelogos.id
          LEFT JOIN {$ASL_PREFIX}stores_categories ON s.`id` = {$ASL_PREFIX}stores_categories.store_id
          WHERE ".$where_clause.';';

    $result = $wpdb->get_results($query);

    //  When we have a store in result
    if($result) {
      return $result[0];
    }

    return null;
  }

  /**
   * [get_closest_store Get the closest store within the given distance]
   * @param  [type] $origLat [description]
   * @param  [type] $origLon [description]
   * @param  [type] $dist    [description]
   * @return [type]          [description]
   */
  public static function get_closest_store($origLat, $origLon, $dist) {

    global $wpdb;
    
    $prefix  = $wpdb->prefix."asl_";

    $clause  = 'HAVING distance < '.$dist;

    //  Query the get the closest store
    $query = "SELECT id,title, lat, lng, email,phone, 3956 * 2 * 
              ASIN(SQRT( POWER(SIN(($origLat - lat)*pi()/180/2),2)
              +COS($origLat*pi()/180 )*COS(lat*pi()/180)
              *POWER(SIN(($origLon-lng)*pi()/180/2),2))) 
              as distance FROM {$prefix}stores WHERE 
              lng between ($origLon-$dist/cos(radians($origLat))*69) 
              and ($origLon+$dist/cos(radians($origLat))*69) 
              and lat between ($origLat-($dist/69)) 
              and ($origLat+($dist/69)) $clause
              ORDER BY distance limit 1";

    
    $result = $wpdb->get_results($query);

    //  When we have a store in result
    if($result) {
      return $result[0];
    }

    return null;
  }

  /**
   * [_get_custom_fields Method to Get the Custom Fields]
   * @return [type] [description]
   */
  public static function get_custom_fields() {

    global $wpdb;
    
    //  Fields
    $fields = $wpdb->get_results("SELECT content FROM ".ASL_PREFIX."settings WHERE `type` = 'fields'");
    $fields = ($fields && isset($fields[0]))? json_decode($fields[0]->content, true): [];

    return $fields;
  }

  /**
   * [generate_tmpl_css Generate the CSS codes for the template]
   * @param  [type] $template [description]
   * @return [type]           [description]
   */
  public static function generate_tmpl_css($template) {

    $codes_json = self::get_setting('ui-template', 'template-'.$template);


    if($codes_json) {

      $color_codes  = json_decode($codes_json, true);
      $class_prefix = ($template == 'wc')? '.asl-cont': '#asl-storelocator.asl-cont.asl-template';
      $color_html   = $class_prefix."-{$template} {";

      foreach ($color_codes as $key => $code) {
        
        if($code) {

          if(is_numeric($code)) {
            $code = $code.'px';
          }

          $color_html  .= "--sl-{$key}: {$code};";
        }
      }
      $color_html .= "}";
      
      return $color_html;
    }

    return '';
  }

  /**
   * [set_setting Set the setting value]
   * @param [type] $_value [description]
   * @param [type] $type   [description]
   */
  public static function set_setting($_value, $type, $name = '') {

    global $wpdb;

    $prefix = ASL_PREFIX;

    $type   = esc_sql($type);

    $sql    = "SELECT count(*) AS 'count' FROM {$prefix}settings WHERE `type` = '{$type}'";

    //  if  have name?
    if($name) {

      $name = esc_sql($name);

      $sql .= " AND `name` = '$name'";
    }

    // To check if the settings exist?  
    $c = $wpdb->get_results($sql);

    $data_params = array('content' => $_value, 'type'=> $type, 'name' => $name);

    //  if exist? Update it
    if($c[0]->count  >= 1) {

      $where_clause  = array('type' => $type);

      //  Add name in where clause
      if($name) {
        $where_clause['name'] = $name;        
      }

      return $wpdb->update($prefix."settings", $data_params, $where_clause);
    }
    //  Create new value
    else {
      return $wpdb->insert($prefix."settings", $data_params);
    }

    return null;
  }

  /**
   * [get_setting Get the setting value]
   * @param  [type] $type [description]
   * @return [type]       [description]
   */
  public static function get_setting($type, $name = '') {

    global $wpdb;

    $prefix = ASL_PREFIX;

    $type   = esc_sql($type);

    $sql    = "SELECT * FROM {$prefix}settings WHERE `type` = '{$type}'";

    //  if  have name?
    if($name) {

      $name = esc_sql($name);

      $sql .= " AND `name` = '$name'";
    }

    $setting_row = $wpdb->get_results($sql);

    //  Return the value if exist
    if(isset($setting_row) && isset($setting_row[0])) {

      return $setting_row[0]->content;
    }

    return 0;
  }



  /**
   * [get_config Get the Config value]
   * @param  [type] $type [description]
   * @return [type]       [description]
   */
  public static function get_configs($_columns = '*') {

    global $wpdb;

    $columns      = $_columns;
    $prefix       = ASL_PREFIX;

    //  Where clause
    $where_clause = '';

    //  For limited columns
    if(is_array($columns)) {

      array_walk($columns, function(&$x) {$x = "'$x'";});
      
      $columns      = implode(',', $columns);

      $where_clause = " WHERE `key` IN ({$columns});";
    }
    else if($columns != '*') {

      $columns      = esc_sql($columns);

      $where_clause = " WHERE `key` = '$columns';";
    }
    else {

      $where_clause = ";";
    }

    //  Fetch the configs
    $configs     = $wpdb->get_results("SELECT * FROM {$prefix}configs".$where_clause);

    //  Config to return
    $all_configs = array();
    
    //  Loop over the config
    foreach($configs as $_config)
      $all_configs[$_config->key] = $_config->value;


    //  Single value?
    if(is_string($_columns) && $_columns != '*') {
      return $all_configs[$columns];
    }

    return $all_configs;
  }


  /**
   * [getSettings Get the Settings]
   * @param  [type] $type [description]
   * @return [type]       [description]
   */
  public static function getSettings($type) {

    global $wpdb;


    $prefix  = $wpdb->prefix."asl_";

    $results = $wpdb->get_results("SELECT * FROM {$prefix}settings WHERE `type` = '$type'");

    if($results && isset($results[0])) {

      $results = ($results && isset($results[0]))? json_decode($results[0]->content, true): [];

      return $results;
    } 

    return null;
  }


  /**
   * [getLangControl Render a Lang Control to select lang]
   * @return [type] [description]
   */
  public static function getLangControl($json = false) {    

    return '';
  }

  /**
   * [slugify Create Slug]
   * @param  [type] $string [description]
   * @return [type]         [description]
   */
  public static function slugify($store) {

    $string = $store['title'].'-'.$store['city'];

    $string = strtolower(trim(preg_replace('/[^A-Za-z0-9-]+/', '-', $string), '-'));

    return preg_replace('/-+/', '-', $string);
  }

  /**
   * [getLnt Get the Coordinates]
   * @param  [type]  $_address [description]
   * @param  [type]  $key      [description]
   * @param  boolean $debug    [description]
   * @return [type]            [description]
   */
  public static function getLnt($_address,$key,$debug = false) {

    $response = new \stdclass();
    $response->success = false;

    $url = "https://maps.googleapis.com/maps/api/geocode/json?address=".urlencode($_address);

    if($key) {
      $url .= '&key='.$key;
    }

    $error_message = null;
    $result        = wp_remote_request($url);
    

    //Debug
    if($debug) {

      return $result;
    }

    if($result) {

      if(is_object($result)) {
        //  Failed to fetch records
        return array();
      }
      else if(isset($result['body'])) {

        $result   = json_decode($result['body'], true);

        if(isset($result['results'][0])) {

          $result1=$result['results'][0];

          $result1 = array(
            'address'=> $result1['formatted_address'],
            'lat' => $result1['geometry']['location']['lat'],
            'lng' => $result1['geometry']['location']['lng']
          );
          return $result1;
        }
        else
          return array();
      }
    }

    return array();
  }


  /**
   * [validate_coordinate Check if the Coordinates are correct]
   * @return [type] [description]
   */
  public static function validate_coordinate($lat, $lng) {

      if($lat && $lng && is_numeric($lat) && is_numeric($lng)) {

          if ($lat < -90 || $lat > 90) {
            return false;
          }


          if ($lng < -180 || $lng > 180) {
            return false;
          }

          return true;
      }

      return false;
  }


  /**
   * [create_upload_dirs Create the upload directories if not exist]
   * @return [type] [description]
   */
  public static function create_upload_dirs() {

    //  CREATE DIRECTORY IF NOT EXISTS
    if(!file_exists(ASL_UPLOAD_DIR)) {
      mkdir( ASL_UPLOAD_DIR, 0775, true );
    }

    //  4 folders to copy
    $folders_to_copy = array('icon', 'Logo', 'svg', 'kml');

    //  Create sub-directories
    foreach ($folders_to_copy as $folder) {

      //  CREATE DIRECTORY IF NOT EXISTS
      if(!file_exists(ASL_UPLOAD_DIR.$folder.'/')) {

        mkdir( ASL_UPLOAD_DIR.$folder.'/', 0775, true );
      }
    }
  }


  /**
   * [tmpl_name Get the full name of the template by id]
   * @param  [type] $template [description]
   * @return [type]           [description]
   */
  private static function tmpl_name($template) {

    switch ($template) {
      
      case '0':
      case '1':
      case '2':
      case '3':
      case 'list':
        
        $template = 'template-frontend-'.$template.'.php';
        break;

      case 'search':

        $template = 'asl-search.php';
        break;

      case 'form':

        $template = 'asl-store-form.php';
        break;

      case 'store':

        $template = 'asl-store-page.php';
        break;

      case 'lead':

        $template = 'asl-lead-form.php';
        break;

      default:
        
        $template = null;
        break;
    }


    return $template;
  }

  /**
   * [backup_template Copy the template file into theme Directory]
   * @param  [type] $template_id [description]
   * @return [type]              [description]
   */
  public static function backup_template($template_id) {
    
    $template_name      = self::tmpl_name($template_id);

    //  Validate the name
    if(!$template_name) {
      return ['success' => false, 'msg' => esc_attr__('Error! Template file is not valid.', 'asl_admin')];
    }

    //$theme_directory = get_template_directory();
    $theme_directory = get_stylesheet_directory();

    $template_file_path = ASL_PLUGIN_PATH.'public/partials/'.$template_name;

    
    if (@copy($template_file_path, $theme_directory.'/'.$template_name)) {

      return ['success' => true, 'msg' => esc_attr__('Template file moved to theme directory.', 'asl_admin')];
    }

    return ['success' => false, 'msg' => esc_attr__('Error! fail to move the file, check permisions.', 'asl_admin')];
  }


  /**
   * [backup_template Remove the template file from theme Directory]
   * @param  [type] $template_id [description]
   * @return [type]              [description]
   */
  public static function remove_template($template_id) {
    
    $template_name      = self::tmpl_name($template_id);

    //  Validate the name
    if(!$template_name) {
      return ['success' => false, 'msg' => esc_attr__('Error! Template file is not valid.', 'asl_admin')];
    }

    //$theme_directory = get_template_directory();
    $theme_directory = get_stylesheet_directory();

    
    if(!file_exists($theme_directory.'/'.$template_name)) {

      return ['success' => false, 'msg' => esc_attr__("Error! Template file doesn't exist.", 'asl_admin')];
    }

    //$template_file_path = ASL_PLUGIN_PATH.'public/partials/'.$template_name;

    if (@unlink($theme_directory.'/'.$template_name)) {

      return ['success' => true, 'msg' => esc_attr__('Template file removed from the theme directory.', 'asl_admin')];
    }

    return ['success' => false, 'msg' => esc_attr__('Error! fail to move the file, check permisions.', 'asl_admin')];
  }


  /**
   * [migrate_assets Migrate the Assets]
   * @return [type] [description]
   */
  public static function migrate_assets() {

    WP_Filesystem();

    $is_copied = false;

    //  Create Directories
    self::create_upload_dirs();

    //  Check for the dependency
    if (!function_exists('copy_dir')) {
      require_once(ABSPATH . 'wp-admin/includes/file.php');
    }

    // Check if get_plugins() function exists. This is required on the front end of the
    if ( ! function_exists( 'get_plugins' ) ) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }

    $plugins = get_plugins();


    //  Current version
    $active_version = explode('/', trim(ASL_PLUGIN_PATH, '/'));
    $active_version = $active_version[count($active_version) - 1];

    foreach ($plugins as $key => $plugin) {
      
      if($plugin['TextDomain']  == 'asl_locator' || $plugin['TextDomain']  == 'agile-store-locator') {

        $sl_path_bath = explode('/', $key);
        $sl_path_bath = $sl_path_bath[0];
        

        if($active_version != $sl_path_bath) {

          //  Copy the Folders
          $folders_to_copy = array('icon', 'Logo', 'svg');

          foreach ($folders_to_copy as $folder) {
            
            $target_dir  = ASL_UPLOAD_DIR.$folder.'/';
            $plugin_path = WP_PLUGIN_DIR.'/'.$sl_path_bath;
            $source_dir  = $plugin_path.'/public/'.$folder.'/';

            @copy_dir($source_dir, $target_dir);

            $is_copied = true;
          }
          
        }
      }
    }

    return $is_copied;
  }


  /**
   * [copy_assets Copy the assets directory to the uploads folder]
   * @return [type] [description]
   */
  public static function copy_assets() {

    WP_Filesystem();

    //  Create Directories
    self::create_upload_dirs();

    //  Check for the dependency
    if (!function_exists('copy_dir')) {
      require_once(ABSPATH . 'wp-admin/includes/file.php');
    }

    //  4 folders to copy
    $folders_to_copy = array('icon', 'Logo', 'svg');

    foreach ($folders_to_copy as $folder) {
      
      $target_dir = ASL_UPLOAD_DIR.$folder.'/';
      $source_dir = ASL_PLUGIN_PATH.'public/'.$folder.'/';

      @copy_dir($source_dir, $target_dir);
    }
  }


  /**
   * [openHours Provide string of Opening Hours]
   * @param  [type] $store [description]
   * @return [type]        [description]
   */
  public static function openHours($store) {

    $timings    = '';

    $days_str   = array('sun'=> esc_attr__( 'Sun','asl_locator'), 'mon'=> esc_attr__('Mon','asl_locator'), 'tue'=>  esc_attr__( 'Tues','asl_locator'), 'wed'=>  esc_attr__( 'Wed','asl_locator' ), 'thu'=> esc_attr__( 'Thur','asl_locator'), 'fri'=> esc_attr__( 'Fri','asl_locator' ), 'sat'=> esc_attr__( 'Sat','asl_locator'));

    if($store->open_hours) {

        $open_hours = json_decode($store->open_hours);

        $timings    = '';

        if(is_object($open_hours)) {

          $open_details = array();

          foreach($open_hours as $key => $_day) {

            $key_value = '';

            if($_day && is_array($_day)) {

              $key_value = implode(',', $_day);

              $timings  .= '<div class="sl-day">';
              $timings  .= '<div class="sl-day-lbl">'.$days_str[$key].'</div><div class="sl-timings">';


              //  Loop Over the Timing
              foreach ($_day as $time) {
                
                $timings .= '<span class="sl-time">'.$time.'</span>';
              }

              $timings  .= '</div></div>';

            }
            else if($_day == '1') {

              $key_value = $_day;
            }
            else  {

              $key_value = '0';
            }

            $open_details[] = $key.'='.$key_value;
          }

          $open_hours_value = implode('|', $open_details);
        }

    }


    return $timings;
  }


  /**
   * [dayFullName Convert the symbol to full name]
   * @param  [type] $day [description]
   * @return [type]      [description]
   */
  public static function dayFullName($day) {

    $days_full = [
      'mon' => 'Monday',
      'tue' => 'Tuesday',
      'wed' => 'Wednesday',
      'thu' => 'Thursday',
      'fri' => 'Friday',
      'sat' => 'Saturday',
      'sun' => 'Sunday'
    ];

    return $days_full[$day];
  }

  /**
   * [googleSchema description]
   * @param  [type] $store_data [description]
   * @return [type]             [description]
   */
  public static function googleSchema($store_data) {

    
    require_once 'Schema/SLSchemaGenerator.php';

    //  Main instance
    $store = new SLSchemaGenerator('Store');

    $store->name($store_data->title);

    if($store_data->email)
      $store->email($store_data->email);

    if(isset($store_data->path) && $store_data->path)
      $store->setProperty('images', [ASL_UPLOAD_URL.'Logo/'.$store_data->path]);

    //  Address
    $address = new SLSchemaGenerator('PostalAddress', false);

    if($store_data->street)
      $address->setProperty('streetAddress', $store_data->street);

    if($store_data->city)
      $address->setProperty('addressLocality', $store_data->city);

    if($store_data->state)
      $address->setProperty('addressRegion', $store_data->state);

    if($store_data->postal_code)
      $address->setProperty('postalCode', $store_data->postal_code);

    if($store_data->country)
      $address->setProperty('addressCountry', $store_data->country);


    //  Geo
    $geo = new SLSchemaGenerator('GeoCoordinates', false);
    $geo->setProperty('latitude', $store_data->lat);
    $geo->setProperty('longitude', $store_data->lng);



    $store->setProperty('address', $address);
    $store->setProperty('geo', $geo);

    //  URL
    if(isset($store_data->website) && $store_data->website)
      $store->setProperty('url', $store_data->website);

    //  Phone
    if($store_data->phone)
      $store->setProperty('telephone', $store_data->phone);

    
    //  Opening hours
    if(isset($store_data->hours) && $store_data->hours) {

      $all_open_hours = [];

      $week_hours = json_decode($store_data->hours);
      

      foreach($week_hours as $day => $day_hours) {

        $day_full = self::dayFullName($day);
        
        //  only process array
        if(!$day_hours || !is_array($day_hours))continue;

        $hours_spec = new SLSchemaGenerator('OpeningHoursSpecification', false);
        $hours_spec->setProperty('dayOfWeek', $day_full);
        $hours_spec->setProperty('opens', $day_full);
  
        //  explode to get hours          
        $open_close = explode('-', $day_hours[0]);

        $open_close = array_map(function($o) { return trim($o); }, $open_close);

        $hours_spec->setProperty('opens', $open_close[0]);
        $hours_spec->setProperty('closes', $open_close[1]);

        $all_open_hours[] = $hours_spec;
      }

      if(count($all_open_hours) > 1) {

        $store->setProperty('openingHoursSpecification', $all_open_hours);
      }
    }


    return $store->toScript();
  }

  /**
   * [fix_backward_compatible Fix the Backward Compatibility]
   * @return [type] [description]
   */
  public static function fix_backward_compatible()
  {
    
    global $wpdb;

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    
    $prefix    = $wpdb->prefix."asl_";
    $table_name  = ASL_PREFIX."stores_timing";
    $store_table = ASL_PREFIX."stores";
    $database    = $wpdb->dbname;

    //Add Open Hours Column   
    $sql  = "SELECT count(*) as c FROM information_schema.COLUMNS WHERE TABLE_NAME = '{$store_table}' AND COLUMN_NAME = 'open_hours';";// AND TABLE_SCHEMA = '{$database}'
    $result = $wpdb->get_results($sql);
    
    if($result[0]->c == 0) {

      $wpdb->query("ALTER TABLE {$store_table} ADD open_hours text;");
    }
    else {
      
      return;
    }


    //Check if Exist
    /*
    if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
      return;
    }
    */
    


    //Convert All Timings
    $stores = $wpdb->get_results("SELECT s.`id` , s.`start_time`, s.`time_per_day` , s.`end_time`, t.* FROM {$store_table} s LEFT JOIN {$table_name} t ON s.`id` = t.`store_id`");
    
    foreach($stores as $timing) {

      $time_object = new \stdClass();
      $time_object->mon = array();
      $time_object->tue = array();
      $time_object->wed = array();
      $time_object->thu = array();
      $time_object->fri = array();
      $time_object->sat = array();
      $time_object->sun = array();
      

      if($timing->time_per_day == '1') {

        if($timing->start_time_0 && $timing->end_time_0) {

          $time_object->sun[] = $timing->start_time_0 .' - '. $timing->end_time_0;
        }

        if($timing->start_time_1 && $timing->end_time_1) {

          $time_object->mon[] = $timing->start_time_1 .' - '. $timing->end_time_1;
        }

        if($timing->start_time_2 && $timing->end_time_2) {

          $time_object->tue[] = $timing->start_time_2 .' - '. $timing->end_time_2;
        }


        if($timing->start_time_3 && $timing->end_time_3) {

          $time_object->wed[] = $timing->start_time_3 .' - '. $timing->end_time_3;
        }

        if($timing->start_time_4 && $timing->end_time_4) {

          $time_object->thu[] = $timing->start_time_4 .' - '. $timing->end_time_4;
        }

        if($timing->start_time_5 && $timing->end_time_5) {

          $time_object->fri[] = $timing->start_time_5 .' - '. $timing->end_time_5;
        }

        if($timing->start_time_6 && $timing->end_time_6) {

          $time_object->sat[] = $timing->start_time_6 .' - '. $timing->end_time_6;
        }
      }
      else if(trim($timing->start_time) && trim($timing->end_time)) {

        $time_object->mon[] = $time_object->sun[] = $time_object->tue[] = $time_object->wed[] = $time_object->thu[] =$time_object->fri[] = $time_object->sat[] = trim($timing->start_time) .' - '. trim($timing->end_time);
      }
      else {

        $time_object->mon = $time_object->tue = $time_object->wed = $time_object->thu = $time_object->fri = $time_object->sat = $time_object->sun = '1';
      }
      
      $time_object = json_encode($time_object);

      //Update new timings
      $wpdb->update(ASL_PREFIX."stores",
        array('open_hours'  => $time_object),
        array('id' => $timing->id)
      );
    }


    $sql = "DROP TABLE IF EXISTS `".$table_name."`;";
    $wpdb->query( $sql );
  }

  /**
   * [getaddress Get the Address]
   * @param  [type] $lat [description]
   * @param  [type] $lng [description]
   * @return [type]      [description]
   */
  public static function getaddress($lat,$lng) {

    $url = 'http://maps.googleapis.com/maps/api/geocode/json?latlng='.trim($lat).','.trim($lng);

    $json = @file_get_contents($url);
    $data=json_decode($json);
    $status = $data->status;
    if($status=="OK")
    return $data->results[0]->formatted_address;
    else
    return false;
  }


  /**
   * [getCoordinates Get the Coordinates]
   * @param  [type] $street  [description]
   * @param  [type] $city    [description]
   * @param  [type] $state   [description]
   * @param  [type] $zip     [description]
   * @param  [type] $country [description]
   * @param  [type] $key     [description]
   * @return [type]          [description]
   */
  public static function getCoordinates($street,$city,$state,$zip,$country,$key)
  {
    $params = array(
      'address' => $street,'city'=> $city, 'state'=> $state,'postcode'=> $zip, 'country' => $country
    );

    if($params['postcode'] || $params['city'] || $params['state']) {

      //$_address = $params['address'].', '.$params['postcode'].'  '.$params['city'].' '.$params['state'].' '.$params['country'];
      $_address = (($params['address'])? $params['address'].', ':'').(($params['postcode'])? $params['postcode'].', ':'').(($params['city'])? $params['city'].', ':'').' '.(($params['state'])? $params['state'].', ':'').$params['country'];
      
      $response = self::getLnt($_address, $key);
      
      if(/*$response['address'] && */isset($response['lng']) && $response['lng'] && isset($response['lat']) && $response['lat']) {
        
        return $response;
      }
      else {
        return null;
      }
    }
    else
    {
      return null;
    }
    
    return true;
  }


  /**
   * [create_zip Create a Zip File]
   * @param  array   $files       [description]
   * @param  string  $destination [description]
   * @param  boolean $overwrite   [description]
   * @return [type]               [description]
   */
  public static function create_zip($files = array(),$destination = '',$overwrite = false) {
    
    //if the zip file already exists and overwrite is false, return false
    if(file_exists($destination) && !$overwrite) { return false; }
    

    //vars
    $valid_files = array();
    //if files were passed in...
    if(is_array($files)) {
      //cycle through each file
      foreach($files as $file) {

        //make sure the file exists
        if(file_exists($file)) {
          $valid_files[] = $file;
        }
      }
    }

    //if we have good files...
    if(count($valid_files)) {
      //create the archive
      $zip = new ZipArchive();
      //if($zip->open($destination,$overwrite ? ZIPARCHIVE::OVERWRITE : ZIPARCHIVE::CREATE) !== true) {
      if($zip->open($destination,$overwrite ? ZIPARCHIVE::OVERWRITE : ZIPARCHIVE::CREATE) !== true) {
        
        return false;
      }

      //add the files
      foreach($valid_files as $file) {

        $relativePath = str_replace(ASL_UPLOAD_DIR, '', $file);
        $zip->addFile($file,$relativePath);
      }
      //debug
      //echo 'The zip archive contains ',$zip->numFiles,' files with a status of ',$zip->status;
      
      //close the zip -- done!
      $zip->close();
      
      //check to make sure the file exists
      return file_exists($destination);
    }
    else
    {
      return false;
    }
  }


  /**
   * [extract_assets Extract a Zip FIle]
   * @param  [type] $zip_path [description]
   * @return [type]           [description]
   */
  public static function extract_assets($zip_path) {

    if(!file_exists($zip_path)) {
      return false;
    }

    $zip = new ZipArchive();
    
    if ($zip->open($zip_path) === true) {

        $allow_exts = array('jpg','png','jpeg','JPG','gif','svg','SVG');  
        
        for($i = 0; $i < $zip->numFiles; $i++) {
            
          $a_file = $zip->getNameIndex($i);
          
          $extension  = explode('.', $a_file);
          $extension  = $extension[count($extension) - 1];

          //Extract only allowed extension
          if(in_array($extension, $allow_exts)) {

            //$zip->extractTo(ASL_PLUGIN_PATH.'public', array($a_file));
            $zip->extractTo(ASL_UPLOAD_DIR, array($a_file));
          }
        }  

        //Close the connection                 
        $zip->close();   

        return true;                
    }

    return false;
  }


  /**
   * [get_template_views Will return both infobox and list in array]
   * @param  [type] $template_id [description]
   * @return [type]              [description]
   */
  public static function get_template_views($template_id) {

    global $wpdb;

    //  when id is missing
    if(!$template_id)
      $template_id = '0';

    //  Template ID
    $template_name = 'template-'.$template_id;

    $results       = $wpdb->get_results($wpdb->prepare("SELECT `type`,`content` FROM ".ASL_PREFIX."settings WHERE `name` = %s", $template_name), ARRAY_A );

    $contents = array('list' => null , 'infobox' => null);

    //  Get the templates
    foreach($results as $row) {
      $contents[$row['type']] = $row['content'];
    }


    //  Loop over the types for operations
    foreach (['list','infobox'] as $tmpl_type) {

      //  No infobox for List layout
      if($template_id == 'list' && $tmpl_type == 'infobox')
        continue; 
      
      $content = $contents[$tmpl_type];

      //  When HTML is missing from the data, include from files
      if(empty($contents[$tmpl_type])) {

        ob_start();
        // include simple products HTML
        include ASL_PLUGIN_PATH.'public/views/template-'.$template_id.'-'.$tmpl_type.'.html';

        $content = ob_get_contents();
        
        //  clean it
        ob_end_clean();
      }


      //  Add the translation
      preg_match_all("/\[[^\]]*\]/", $content, $translate_words);

      //  Brackets to replace
      $brackets = array('[',']');

      //  Get the first index
      if($translate_words && isset($translate_words[0]) && is_array($translate_words[0])) {
        $translate_words = $translate_words[0];
      }

      //  Loop for Translation Words
      foreach ($translate_words as $word) {
          
        $word_text  = str_replace($brackets, '', $word);

        if($word_text) {
          $content    = str_replace($word, esc_attr__($word_text, 'asl_locator'), $content);
        }
      }

      //  Replace the Path URL
      $content    = str_replace('{{:URL}}', ASL_UPLOAD_URL , $content);

      //  put it back to array
      $contents[$tmpl_type] = $content;
    }
    

    return $contents;
  }

  /**
   * [getTemplateView get template by id]
   * @param  [type] $template_id [description]
   * @return [type]              [description]
   */
  public static function getTemplateView($template_id) {
    
    global $wpdb;


    if(!$template_id)
      $template_id = '0';


    $template_id = 'template-'.$template_id;

    $html = array();
    $prev = $wpdb->get_results($wpdb->prepare("SELECT COUNT('name') as 'count' FROM ".ASL_PREFIX."settings WHERE `name` = %s", $template_id));


    if($prev[0]->count  >= 1) {

      //  Template Query 
       $html = $wpdb->get_results($wpdb->prepare("SELECT `type`,`content` FROM ".ASL_PREFIX."settings WHERE `name` = %s", $template_id)  ,ARRAY_A );

       if ($html) $html[0];
    }else{

      $sections = array('list','infobox');
      

      foreach ($sections as $key => $section) {
        $html[$key]['type'] = $section;

        //  open stream
        ob_start();
        // include simple products HTML
        include ASL_PLUGIN_PATH.'public/views/'.$template_id.'-'.$section.'.html';

        $html[$key]['content'] = ob_get_contents();
        
        //  clean it
        ob_end_clean();
        
      }
      
    }

     return $html;
  }


  /**
   * [removeFile Remove a file]
   * @param  [type] $file      [description]
   * @param  [type] $directory [description]
   * @return [type]            [description]
   */
  public static function removeFile($file, $directory) {

    $response = new \stdclass();

    //  make file path
    $file_path = $directory.$file;

    //  Validate the file?
    if(file_exists($file_path)) { 
          
      unlink($file_path);
    
      $response->success  = true;
      $response->msg      = esc_attr__('File deleted successfully.','asl_admin');
    }
    else
      $response->error = esc_attr__('Error! fail to delete the file.','asl_admin');

    return $response;
  }

}

?>
