<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://agilelogix.com
 * @since      1.0.0
 *
 * @package    AgileStoreLocator
 * @subpackage AgileStoreLocator/public
 */

/**
 * The public-facing functionality of the plugin is for the AJAX Requests.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    AgileStoreLocator
 * @subpackage AgileStoreLocator/frontend
 * @author     AgileLogix <support@agilelogix.com>
 */
class AgileStoreLocator_Request {

	/**
	 * [load_stores Load the Stores using AJAX Request]
	 * @return [type] [description]
	 */
	public function load_stores($output_return = false, $_lang = null) {

		global $wpdb;

		$nonce = isset($_GET['nonce'])? $_GET['nonce']: null;
		
		/*
		if ( ! wp_verify_nonce( $nonce, 'asl_remote_nonce' ))
 			die ( 'CRF check error.');
 		*/
		$load_all 	 = (isset($_GET['load_all']) && $_GET['load_all'] == '1' || $output_return)?	true:false;
		$accordion   = (isset($_GET['layout']) && $_GET['layout'] == '1')?true:false;
		$category    = (isset($_GET['category']))?$_GET['category']:null;
		$brand    	 = (isset($_GET['brand']))?$_GET['brand']:null;
		$special     = (isset($_GET['special']))?$_GET['special']: null;
		$stores      = (isset($_GET['stores']))?$_GET['stores']:null;
		$lang      	 = (isset($_GET['lang']))?$_GET['lang']: '';

		//	Link type we replace the website with the slug
		$slug_link   = (isset($_GET['slug_link']))?true:false;

		$ASL_PREFIX  = ASL_PREFIX;

		$bound   				= '';

		$extra_sql 			= '';
		$country_field 	= '';

		//	Cache Lang
		if($_lang) {
			$lang = $_lang;
		}
		

		$country_field = " {$ASL_PREFIX}countries.`country`,";
   	$extra_sql 		 = "LEFT JOIN {$ASL_PREFIX}countries ON s.`country` = {$ASL_PREFIX}countries.id";
    

    $clause = '';

    if($category) {

			$load_categories = explode(',', $category);
			$the_categories  = array();

			foreach($load_categories as $_c) {

				//	Clean it
				if(is_numeric($_c)) {
					$the_categories[] = $_c;
				}
			}

			$the_categories  = implode(',', $the_categories);
			$category_clause = " AND id IN (".$the_categories.')';
			$clause 		     = " AND {$ASL_PREFIX}stores_categories.`category_id` IN (".$the_categories.")";
		}


       	///if marker param exist
		if($stores) {

			$stores = explode(',', $stores);

			//only number
			$store_ids = array();
			foreach($stores as $m) {

				if(is_numeric($m)) {
					$store_ids[] = $m;
				}
			}

			if($store_ids) {

				$store_ids = implode(',', $store_ids);
				$clause    .= " AND s.`id` IN ({$store_ids})";				
			}
		}



		$query   = "SELECT s.`id`, `title`,  `description`, `street`,  `city`,  `state`, `postal_code`, {$country_field} `lat`,`lng`,`phone`,  `fax`,`email`,`website`,`logo_id`,{$ASL_PREFIX}storelogos.`path`,`marker_id`,`description_2`,`open_hours`, `ordr`,`brand`,`special`, `custom`,`slug`,
					group_concat(category_id) as categories FROM {$ASL_PREFIX}stores as s 
					LEFT JOIN {$ASL_PREFIX}storelogos ON logo_id = {$ASL_PREFIX}storelogos.id
					LEFT JOIN {$ASL_PREFIX}stores_categories ON s.`id` = {$ASL_PREFIX}stores_categories.store_id
					$extra_sql
					WHERE (s.`pending` IS NULL OR s.`pending` = '') AND s.`lang` = '$lang' AND (is_disabled is NULL || is_disabled = 0) AND (`lat` != '' AND `lng` != '') {$bound} {$clause}
					GROUP BY s.`id` ORDER BY `title` ";

		$query .= " LIMIT 10000";

		
		$all_results = $wpdb->get_results($query);


		$debug_error = true;

		if($debug_error) {

			$err_message = isset($wpdb->last_error)? $wpdb->last_error: null;
			
			if(!$all_results && $err_message) {

				$database = $wpdb->dbname;

				//  Check if the new columns are there or not
	      $sql  = "SELECT count(*) as c FROM information_schema.COLUMNS WHERE TABLE_NAME = '{$ASL_PREFIX}stores' AND COLUMN_NAME = 'lang' AND TABLE_SCHEMA = '{$database}'";
	      $col_check_result = $wpdb->get_results($sql);
	      
	      if($col_check_result[0]->c == 0) {
	          
	          require_once ASL_PLUGIN_PATH . 'includes/class-agile-store-locator-activator.php';
	          
	          //  Run the script to add missing tables
	          AgileStoreLocator_Activator::activate();
	      }

				echo json_encode([$err_message]);die;
			}
		}


		$days_in_words 	= array('sun'=> esc_attr__( 'Sun','asl_locator'), 'mon'=> esc_attr__('Mon','asl_locator'), 'tue'=> esc_attr__( 'Tues','asl_locator'), 'wed'=> esc_attr__( 'Wed','asl_locator' ), 'thu'=> esc_attr__( 'Thur','asl_locator'), 'fri'=> esc_attr__( 'Fri','asl_locator' ), 'sat'=> esc_attr__( 'Sat','asl_locator'));
		$days 		   		= array('mon','tue','wed','thu','fri','sat','sun');


		//	Only fetch the config when link type is set to rewrite
		$slug_url = '';

		if($slug_link) {

			$rewrite_config = AgileStoreLocator_Helper::get_configs(['rewrite_slug', 'rewrite_id']);

			if(isset($rewrite_config['rewrite_slug']) && $rewrite_config['rewrite_slug'] && $rewrite_config['rewrite_id']) {

				$slug_url = '/'.$rewrite_config['rewrite_slug'].'/';
			}
			//	rewrite data is incomplete
			else {

				$slug_link = null;
			}
		}


		//	Loop over the rows
		foreach($all_results as $aRow) {

			if($aRow->open_hours) {

				$days_are 	= array();
				$open_hours = json_decode($aRow->open_hours);

				foreach($days as $day) {

					if(!empty($open_hours->$day)) {

						$days_are[] = $days_in_words[$day];
					}
				}

				$aRow->days_str = implode(', ', $days_are);
			}


			//	Decode the Custom Fields
			if($aRow->custom) {

				$custom_fields = json_decode($aRow->custom, true);

				if($custom_fields && is_array($custom_fields) && count($custom_fields) > 0) {

					foreach ($custom_fields as $custom_key => $custom_value) {
						
						$aRow->$custom_key = $custom_value;
					}
				}
			}

			unset($aRow->custom);
	  }

	  //	To Return the output object
	  if($output_return) {
	  	return $all_results;
	  }

		echo json_encode($all_results);die;
	}



	/**
	 * [lead_request Recieve a lead form details, that will be sent to the dealer]
	 * @return [type] [description]
	 */
	public function lead_request() {

		global $wpdb;

		$response 					= new \stdclass();
    $response->success 	= false;

		
		$nonce = isset($_POST['vkey'])?$_POST['vkey']:null;

		//	Get the Form Data
 		$lead_details  = isset($_POST['form_params'])? stripslashes_deep($_POST['form_params']): null;

 		//	Separate handling for the store locator contact form
		if($lead_details && isset($lead_details['asl_request_type']) && $lead_details['asl_request_type'] == 'sl-contact') {

			return $this->_contact_form_request($lead_details);
		}


		//	verify it
		if (!$nonce || !wp_verify_nonce( $nonce, 'asl_store_form_nonce' )) {

 			echo json_encode(['success' => false, 'message' => esc_attr__('Error! Invalid lead contact request.','asl_locator')]);die;
		}

 	

 		//	Return false, Must be an array
 		if(!is_array($lead_details)) {

 			echo json_encode(['success' => false, 'message' => esc_attr__('Error! Invalid contact details.','asl_locator')]);die;
 		}

 		//	Validate the request parameter
 		if(!$lead_details['name'] || !$lead_details['email'] || !$lead_details['phone'] || !$lead_details['postal_code']) {

 			$response->message = esc_attr__('Error! Please fill up the stores parameters.','asl_locator');
			echo json_encode($response);die;
 		}

 		$lead_config = isset($_POST['config'])? stripslashes_deep($_POST['config']): null;

 		//	Maximum radius to search the store
 		$area_radius  = (isset($lead_config['radius']) && $lead_config['radius'])? floatval($lead_config['radius']): 25;

 		//	Country code to suffix the geocoding request
 		$lead_country = (isset($lead_config['country']) && $lead_config['country'])? $lead_config['country']: '';

 		//	Get the fields data
		$full_name   = strip_tags(sanitize_text_field($lead_details['name']));
		$user_email  = strip_tags(sanitize_text_field($lead_details['email']));
		$postal_code = strip_tags(sanitize_text_field($lead_details['postal_code']));
		$user_phone  = strip_tags(sanitize_text_field($lead_details['phone']));
		$msg_text 	 = strip_tags(sanitize_text_field($lead_details['message']));


 		//1- Get the Coordinates for the provided zipcode
 		$coordinates = AgileStoreLocator_Helper::getCoordinates('', '', '', $postal_code, $lead_country, AgileStoreLocator_Helper::get_configs('server_key'));


 		//2- Calc the closest Store to the fetched zipcode, 2000 miles
		$closest_store = AgileStoreLocator_Helper::get_closest_store($coordinates['lat'], $coordinates['lng'], $area_radius);

		//	Mail config
		$mail_configs = AgileStoreLocator_Helper::get_configs(['admin_notify', 'notify_email']);

 		//3- Send notification to the dealer, make sure each dealer to recieve only 1 email per day
 		if(($closest_store && $closest_store->email) || $mail_configs['notify_email']) {

 			//	Insert into the leads
			if($wpdb->insert(ASL_PREFIX."leads", array(
				'name' 	=> $full_name,
				'email' => $user_email,
				'postal_code' => $postal_code,
				'phone' 	=> $user_phone,
				'message' => $msg_text
			))) {


				//	Send the lead email notification, when notification is enabled
				if($mail_configs['admin_notify']) {


					$subject  = esc_attr__("Store Locator Lead Request",'asl_locator');

				  $content 	= '<p>'.esc_attr__('A new lead is requesting details about the store: ', 'asl_locator').(($closest_store)?$closest_store->title:''). '</p><br>'.
				  						'<p>'.esc_attr__('Full Name: ','asl_locator'). $full_name.'</p>'.
				  						'<p>'.esc_attr__('Email Address: ','asl_locator').'<a href="mailto:'.$user_email.'">'.$user_email.'</a>'.'</p>'.
				  						'<p>'.esc_attr__('Phone: ','asl_locator').'<a href="tel:'.$user_phone.'">'.$user_phone.'</a>'.'</p>'.
				  						'<p>'.esc_attr__('Postal Code: ','asl_locator'). $postal_code.'</p>';

				  //	When we have message text
				 	if($msg_text) {

				 		$content 	.= '<p>'.esc_attr__('Message: ','asl_locator'). $msg_text.'</p>';
				 	}


				 	// CC headers
				 	$headers  = [];

				 	//	main email sent to the email
				 	$to_email = ($closest_store && $closest_store->email)? $closest_store->email: $mail_configs['notify_email'];


				 	//	Add CC to the admin email when we have a two recievers
				 	if($mail_configs['notify_email'] && $to_email != $mail_configs['notify_email']) {
				 		$headers[] = 'Cc: '.$mail_configs['notify_email'];
				 	}

				  //	Send a email notification
					$response->s = AgileStoreLocator_Helper::send_email($to_email, $subject, $content, $headers);
				}


				$response->message = esc_attr__('Thank you! we have received your request.','asl_locator');
				$response->success = true;
			}
 		}
 		else {

 			$response->message = esc_attr__('Error! Sorry there are no close store to your region.','asl_locator');
 		}

 		echo json_encode($response);
		die;
	}


	/**
	 * [_contact_form_request Handle the request by the contact form in store locator, email, message, rating, id]
	 * @param  [type] $lead_details [description]
	 * @return [type]               [description]
	 */
	private function _contact_form_request($lead_details) {

		global $wpdb;

		$response 					= new \stdclass();
    $response->success 	= false;

		$nonce = isset($_POST['vkey'])?$_POST['vkey']:null;

		//	verify the request
		if (!$nonce || !wp_verify_nonce( $nonce, 'asl_remote_nonce' )) {

 			echo json_encode(['success' => false, 'message' => esc_attr__('Error! Invalid contact request.','asl_locator')]);die;
		}

		//	Return false, Must be an array
 		if(!is_array($lead_details)) {

 			echo json_encode(['success' => false, 'message' => esc_attr__('Error! Invalid contact details.','asl_locator')]);die;
 		}

 		//	Validate the request parameter
 		if(!$lead_details['email'] || !$lead_details['id']) {

 			$response->message = esc_attr__('Error! Please fill up the stores parameters.','asl_locator');
			echo json_encode($response);die;
 		}

 	
	//	Get the fields data
		$store_id    = strip_tags(sanitize_text_field($lead_details['id']));
		$full_name   = strip_tags(sanitize_text_field($lead_details['name']));
		$user_email  = strip_tags(sanitize_text_field($lead_details['email']));
		$user_rating = (isset($lead_details['rating']))? strip_tags(sanitize_text_field($lead_details['rating'])): null;
		$msg_text 	 = strip_tags(sanitize_text_field($lead_details['message']));


		//	Once in 24 hours
		$count = $wpdb->get_results( $wpdb->prepare('SELECT COUNT(*) AS c FROM `'.ASL_PREFIX.'leads` WHERE (created_on > NOW() - INTERVAL 24 HOUR) AND email = %s',
			$user_email
		));

		//	Already submited?
		if(isset($count[0]->c) && $count[0]->c > 0) {

			echo json_encode(['success' => false, 'message' => esc_attr__('Sorry! you have already submited the form.','asl_locator')]);die;
		}

		//	Get the store
		$selected_store = AgileStoreLocator_Helper::get_store(intval($store_id));

		//	Mail config
		$mail_configs 	= AgileStoreLocator_Helper::get_configs(['admin_notify', 'notify_email']);

 		//3- Send notification to the dealer, make sure each dealer to recieve only 1 email per day
 		if(($selected_store && $selected_store->email) || $mail_configs['notify_email']) {

 			//	Insert into the leads
			if($wpdb->insert(ASL_PREFIX."leads", array(
				'name' 	=> $full_name,
				'email' => $user_email,
				'store_id' => $store_id,
				'rating' 	=> $user_rating,
				'message' => $msg_text
			))) {


				//	Send the lead email notification, when notification is enabled
				if($mail_configs['admin_notify']) {


					$subject  = esc_attr__("Store Locator Lead Request",'asl_locator');

				  $content 	= '<p>'.esc_attr__('A new lead is requesting details about the store: ', 'asl_locator').(($selected_store)? $selected_store->title:''). '</p><br>'.
				  						'<p>'.esc_attr__('Full Name: ','asl_locator'). $full_name.'</p>'.
				  						'<p>'.esc_attr__('Email Address: ','asl_locator').'<a href="mailto:'.$user_email.'">'.$user_email.'</a>'.'</p>'.
				  						'<p>'.esc_attr__('Rating: ','asl_locator'). $user_rating.'</p>';

				  //	When we have message text
				 	if($msg_text) {

				 		$content 	.= '<p>'.esc_attr__('Message: ','asl_locator'). $msg_text.'</p>';
				 	}


				 	// CC headers
				 	$headers  = [];

				 	//	main email sent to the email
				 	$to_email = $mail_configs['notify_email'];
				 	$cc_email = ($selected_store && $selected_store->email)? $selected_store->email: null;


				 	//	Send to store when notifer is missing
				 	if(!$to_email) {

				 		$to_email = $cc_email;
				 	}
				 	//	Add CC to the admin email when we have a two recievers
				 	else if($cc_email) {
				 		$headers[] = 'Cc: '.$cc_email;
				 	}

				  //	Send a email notification
					$response->s = AgileStoreLocator_Helper::send_email($to_email, $subject, $content, $headers);
				}


				$response->message = esc_attr__('Thank you! we have received your request.','asl_locator');
				$response->success = true;
			}
 		}
 		else {

 			$response->message = esc_attr__('Error! Sorry there are no close store to your region.','asl_locator');
 		}

 		echo json_encode($response);
		die;
	}


	/**
   * [fixURL Add https:// to the URL]
   * @param  [type] $url    [description]
   * @param  string $scheme [description]
   * @return [type]         [description]
   */
  private function fixURL($url, $scheme = 'http://') {

    if(!$url)
      return '';

    return parse_url($url, PHP_URL_SCHEME) === null ? $scheme . $url : $url;
  }


	/**
	 * [debug Private debug]
	 * @param  [type] $data [description]
	 * @return [type]       [description]
	 */
	private function debug($data) {

		echo '<pre>';
		print_r($data);
		echo '</pre>';
		die;
	}

}
